@@ -34,7 +34,7 @@ from dbus.exceptions import DBusException
 def method(dbus_interface, in_signature=None, out_signature=None,
         async_callbacks=None,
         sender_keyword=None, path_keyword=None, destination_keyword=None,
-        message_keyword=None,
+        message_keyword=None, connection_keyword=None,
         utf8_strings=False, byte_arrays=False):
     """Factory for decorators used to mark methods of a `dbus.service.Object`
     to be exported on the D-Bus.
@@ -86,16 +86,30 @@ def method(dbus_interface, in_signature=None, out_signature=None,
             case of "fallback paths" you'll usually want to use the object
             path in the method's implementation.
 
+            :Since: 0.80.0?
+
         `destination_keyword` : str or None
             If not None (the default), the decorated method will receive
             the destination bus name as a keyword argument with this name.
             Included for completeness - you shouldn't need this.
 
+            :Since: 0.80.0?
+
         `message_keyword` : str or None
             If not None (the default), the decorated method will receive
             the `dbus.lowlevel.MethodCallMessage` as a keyword argument
             with this name.
 
+            :Since: 0.80.0?
+
+        `connection_keyword` : str or None
+            If not None (the default), the decorated method will receive
+            the `dbus.connection.Connection` as a keyword argument
+            with this name. This is generally only useful for objects
+            that are available on more than one connection.
+
+            :Since: 0.82.0
+
         `utf8_strings` : bool
             If False (default), D-Bus strings are passed to the decorated
             method as objects of class dbus.String, a unicode subclass.
@@ -107,6 +121,8 @@ def method(dbus_interface, in_signature=None, out_signature=None,
             This option does not affect object-paths and signatures, which
             are always 8-bit strings (str subclass) encoded in ASCII.
 
+            :Since: 0.80.0
+
         `byte_arrays` : bool
             If False (default), a byte array will be passed to the decorated
             method as an `Array` (a list subclass) of `Byte` objects.
@@ -115,6 +131,8 @@ def method(dbus_interface, in_signature=None, out_signature=None,
             a `ByteArray`, a str subclass. This is usually what you want,
             but is switched off by default to keep dbus-python's API
             consistent.
+
+            :Since: 0.80.0
     """
     _dbus_bindings.validate_interface_name(dbus_interface)
 
@@ -138,6 +156,8 @@ def method(dbus_interface, in_signature=None, out_signature=None,
             args.remove(destination_keyword)
         if message_keyword:
             args.remove(message_keyword)
+        if connection_keyword:
+            args.remove(connection_keyword)
 
         if in_signature:
             in_sig = tuple(_dbus_bindings.Signature(in_signature))
@@ -156,6 +176,7 @@ def method(dbus_interface, in_signature=None, out_signature=None,
         func._dbus_path_keyword = path_keyword
         func._dbus_destination_keyword = destination_keyword
         func._dbus_message_keyword = message_keyword
+        func._dbus_connection_keyword = connection_keyword
         func._dbus_args = args
         func._dbus_get_args_options = {'byte_arrays': byte_arrays,
                                        'utf8_strings': utf8_strings}
@@ -164,7 +185,8 @@ def method(dbus_interface, in_signature=None, out_signature=None,
     return decorator
 
 
-def signal(dbus_interface, signature=None, path_keyword=None):
+def signal(dbus_interface, signature=None, path_keyword=None,
+           rel_path_keyword=None):
     """Factory for decorators used to mark methods of a `dbus.service.Object`
     to emit signals on the D-Bus.
 
@@ -187,40 +209,98 @@ def signal(dbus_interface, signature=None, path_keyword=None):
             Note that when calling the decorated method, you must always
             pass in the object path as a keyword argument, not as a
             positional argument.
+
+            This keyword argument cannot be used on objects where
+            the class attribute ``SUPPORTS_MULTIPLE_OBJECT_PATHS`` is true.
+
+            :Deprecated: since 0.82.0. Use `rel_path_keyword` instead.
+
+        `rel_path_keyword` : str or None
+            A keyword argument to the decorated method. If not None,
+            that argument will not be emitted as an argument of
+            the signal.
+
+            When the signal is emitted, if the named keyword argument is given,
+            the signal will appear to come from the object path obtained by
+            appending the keyword argument to the object's object path.
+            This is useful to implement "fallback objects" (objects which
+            own an entire subtree of the object-path tree).
+
+            If the object is available at more than one object-path on the
+            same or different connections, the signal will be emitted at
+            an appropriate object-path on each connection - for instance,
+            if the object is exported at /abc on connection 1 and at
+            /def and /x/y/z on connection 2, and the keyword argument is
+            /foo, then signals will be emitted from /abc/foo and /def/foo
+            on connection 1, and /x/y/z/foo on connection 2.
+
+            :Since: 0.82.0
     """
     _dbus_bindings.validate_interface_name(dbus_interface)
+
+    if path_keyword is not None:
+        from warnings import warn
+        warn(DeprecationWarning('dbus.service.signal::path_keyword has been '
+                                'deprecated since dbus-python 0.82.0, and '
+                                'will not work on objects that support '
+                                'multiple object paths'),
+             DeprecationWarning, stacklevel=2)
+        if rel_path_keyword is not None:
+            raise TypeError('dbus.service.signal::path_keyword and '
+                            'rel_path_keyword cannot both be used')
+
     def decorator(func):
         member_name = func.__name__
         _dbus_bindings.validate_member_name(member_name)
 
         def emit_signal(self, *args, **keywords):
+            abs_path = None
+            if path_keyword is not None:
+                if self.SUPPORTS_MULTIPLE_OBJECT_PATHS:
+                    raise TypeError('path_keyword cannot be used on the '
+                                    'signals of an object that supports '
+                                    'multiple object paths')
+                abs_path = keywords.pop(path_keyword, None)
+                if (abs_path != self.__dbus_object_path__ and
+                    not self.__dbus_object_path__.startswith(abs_path + '/')):
+                    raise ValueError('Path %r is not below %r', abs_path,
+                                     self.__dbus_object_path__)
+
+            rel_path = None
+            if rel_path_keyword is not None:
+                rel_path = keywords.pop(rel_path_keyword, None)
+
             func(self, *args, **keywords)
-            object_path = self.__dbus_object_path__
-            if path_keyword:
-                kw = keywords.pop(path_keyword, None)
-                if kw is not None:
-                    if not (kw == object_path
-                            or object_path == '/'
-                            or kw.startswith(object_path + '/')):
-                        raise DBusException('Object path %s is not in the '
-                                            'subtree starting at %s'
-                                            % (kw, object_path))
-                    object_path = kw
 
-            message = _dbus_bindings.SignalMessage(object_path,
-                                                   dbus_interface,
-                                                   member_name)
+            for location in self.locations:
+                if abs_path is None:
+                    # non-deprecated case
+                    if rel_path is None or rel_path in ('/', ''):
+                        object_path = location[1]
+                    else:
+                        # will be validated by SignalMessage ctor in a moment
+                        object_path = location[1] + rel_path
+                else:
+                    object_path = abs_path
 
-            if signature is not None:
+                message = _dbus_bindings.SignalMessage(object_path,
+                                                       dbus_interface,
+                                                       member_name)
                 message.append(signature=signature, *args)
-            else:
-                message.append(*args)
 
-            self._connection.send_message(message)
+                location[0].send_message(message)
+        # end emit_signal
 
         args = inspect.getargspec(func)[0]
         args.pop(0)
 
+        for keyword in rel_path_keyword, path_keyword:
+            if keyword is not None:
+                try:
+                    args.remove(keyword)
+                except ValueError:
+                    raise ValueError('function has no argument "%s"' % keyword)
+
         if signature:
             sig = tuple(_dbus_bindings.Signature(signature))
 
